# -*- coding: utf-8 -*-

import json
import mock
import os
import pytest

from django.test import override_settings
from django.utils.dateparse import parse_date

from passerelle.apps.maelis.models import Maelis, Link
from passerelle.apps.maelis.utils import (
    get_school_year,  week_boundaries_datetimes,  month_range, decompose_event)

from passerelle.utils.jsonresponse import APIError

import utils

pytestmark = pytest.mark.django_db

TEST_BASE_DIR = os.path.join(os.path.dirname(__file__), 'data', 'maelis')


def get_xml_file(filename):
    with open(os.path.join(TEST_BASE_DIR, filename), 'rb') as desc:
        return desc.read()


def get_json_file(filename):
    with open(os.path.join(TEST_BASE_DIR, "%s.json" % filename)) as fd:
        return json.load(fd)


@pytest.fixture
def family_service_wsdl():
    return get_xml_file('FamilyService.wsdl')


@pytest.fixture
def invoice_service_wsdl():
    return get_xml_file('InvoiceService.wsdl')


@pytest.fixture
def activity_service_wsdl():
    return get_xml_file('ActivityService.wsdl')


@pytest.fixture
def connector(db):
    return utils.setup_access_rights(Maelis.objects.create(
        slug='test', login='foo', password='bar',
        base_url='http://www3.sigec.fr/entrouvertws/services/'))


@mock.patch('passerelle.utils.Request.get')
@mock.patch('passerelle.utils.Request.post')
def test_link_unlink(mocked_post, mocked_get, family_service_wsdl,
                     connector, app):
    mocked_get.return_value = mock.Mock(content=family_service_wsdl)
    mocked_post.return_value = mock.Mock(content=get_xml_file('readFamilyByPasswordError.xml'),
                                         status_code=500,
                                         headers={'Content-Type': 'text/xml'})
    assert Link.objects.count() == 0
    resp = app.post_json('/maelis/test/link?NameID=local',
                         params={'family_id': '3264', 'password': 'wrong', 'school_year': '2020'})
    assert resp.json['err'] == 1
    assert resp.json['err_desc'] == 'E204 : Le mot de passe est incorrect'
    assert not resp.json['data']
    assert Link.objects.count() == 0

    resp = app.post_json('/maelis/test/unlink?NameID=local')
    assert resp.json['err']
    assert resp.json['err_desc'] == 'User not linked to family'

    mocked_post.return_value = mock.Mock(content=get_xml_file('readFamilyByPasswordResult.xml'),
                                         status_code=200,
                                         headers={'Content-Type': 'text/xml'})
    resp = app.post_json('/maelis/test/link?NameID=local',
                         params={'family_id': '3264', 'password': '123456'})
    assert resp.json['err'] == 0
    assert resp.json['data']
    assert resp.json['data']['number'] == 3264
    assert Link.objects.count() == 1
    link = Link.objects.get()
    assert link.name_id == 'local'
    assert link.family_id == '3264'

    resp = app.post_json('/maelis/test/unlink?NameID=local')
    assert resp.json['err'] == 0
    assert Link.objects.count() == 0


@mock.patch('passerelle.utils.Request.get')
@mock.patch('passerelle.utils.Request.post')
def test_family_info(mocked_post, mocked_get, family_service_wsdl,
                     connector, app):
    mocked_get.return_value = mock.Mock(content=family_service_wsdl)
    mocked_post.return_value = mock.Mock(content=get_xml_file('readFamily.xml'),
                                         status_code=200,
                                         headers={'Content-Type': 'text/xml'})
    assert Link.objects.count() == 0
    Link.objects.create(resource=connector, family_id='3264', name_id='local')
    resp = app.get('/maelis/test/family-info?NameID=local')
    assert resp.json['data']["number"] == 3264
    assert resp.json['data']['rl1InfoBean']
    assert resp.json['data']['childInfoList']

    resp = app.get('/maelis/test/children-info?NameID=local')
    assert resp.json['data']
    for child in resp.json['data']:
        assert child['id']
        assert child['text']

    resp = app.get('/maelis/test/child-info?NameID=local&childID=21293')
    assert resp.json['data']
    assert resp.json['data']['num'] == '21293'
    assert resp.json['data']['id'] == '21293'

    resp = app.get('/maelis/test/adults-info?NameID=local')
    assert resp.json['data']
    for child in resp.json['data']:
        assert child['id']
        assert child['text']


@mock.patch('passerelle.utils.Request.get')
@mock.patch('passerelle.utils.Request.post')
def test_activity_list(mocked_post, mocked_get, family_service_wsdl,
                       activity_service_wsdl, connector, app):
    mocked_get.side_effect = (
        utils.FakedResponse(content=family_service_wsdl,
                            status_code=200,
                            headers={'Content-Type': 'text/xml'}),
        utils.FakedResponse(content=activity_service_wsdl,
                            status_code=200,
                            headers={'Content-Type': 'text/xml'})
    )
    mocked_post.side_effect = (
        utils.FakedResponse(content=get_xml_file('readFamily.xml'),
                            status_code=200,
                            headers={'Content-Type': 'text/xml'}),
        utils.FakedResponse(content=get_xml_file('readActivityListResponse.xml'),
                            status_code=200,
                            headers={'Content-Type': 'text/xml'})
    )
    Link.objects.create(resource=connector, family_id='3264', name_id='local')
    resp = app.get('/maelis/test/activity-list?NameID=local&personID=21293')
    assert resp.json['data']
    for activity in resp.json['data']:
        assert activity['id']
        assert activity['text']


@mock.patch('passerelle.utils.Request.get')
@mock.patch('passerelle.utils.Request.post')
def test_update_coordinates(mocked_post, mocked_get, family_service_wsdl,
                            connector, app):
    mocked_get.return_value = mock.Mock(content=family_service_wsdl)
    mocked_post.side_effect = (
        utils.FakedResponse(content=get_xml_file('updateCoordinatesResponse.xml'),
                            status_code=200,
                            headers={'Content-Type': 'text/xml'}),
        utils.FakedResponse(content=get_xml_file('updateCoordinatesError.xml'),
                            status_code=200,
                            headers={'Content-Type': 'text/xml'})
    )
    Link.objects.create(resource=connector, family_id='3264', name_id='local')
    resp = app.post_json('/maelis/test/update-coordinates?NameID=local&personID=21293',
                         params={'mail': 'foo@example.com'}
                         )
    assert resp.content is not None

    resp = app.post_json('/maelis/test/update-coordinates?NameID=local&personID=21293',
                         params={'town': 'Paris', 'num': '169'}
                         )
    assert resp.json['err']
    assert resp.json['err_desc'] == 'E16 : Le code postal est obligatoire'


@mock.patch('passerelle.utils.Request.get')
@mock.patch('passerelle.utils.Request.post')
def test_list_invoices(mocked_post, mocked_get, invoice_service_wsdl,
                       connector, app):
    mocked_get.return_value = mock.Mock(content=invoice_service_wsdl)
    mocked_post.return_value = mock.Mock(content=get_xml_file('readInvoicesResponse.xml'),
                                         status_code=200,
                                         headers={'Content-Type': 'text/xml'}
                                         )
    Link.objects.create(resource=connector, family_id='3264', name_id='local')
    resp = app.get('/maelis/test/regie/1/invoices?NameID=local')
    assert resp.json['data']
    for invoice in resp.json['data']:
        assert invoice['display_id']
        assert invoice['label']
        assert invoice['total_amount']
        assert not invoice['paid']


@mock.patch('passerelle.utils.Request.get')
@mock.patch('passerelle.utils.Request.post')
def test_get_invoice_details(mocked_post, mocked_get, invoice_service_wsdl,
                             connector, app):
    mocked_get.return_value = mock.Mock(content=invoice_service_wsdl)
    mocked_post.return_value = mock.Mock(content=get_xml_file('readInvoicesResponse.xml'),
                                         status_code=200,
                                         headers={'Content-Type': 'text/xml'}
                                         )
    Link.objects.create(resource=connector, family_id='3264', name_id='local')
    resp = app.get('/maelis/test/regie/1/invoice/3264-53186?NameID=local')
    assert resp.json['data']
    assert resp.json['data']['label'] == 'FACTURATION SEPTEMBRE 2014'
    assert resp.json['data']['display_id'] == '53186'


@pytest.mark.parametrize('date, schoolyear', [
    ('2020-07-30', 2019),
    ('2020-07-31', 2020),
])
def test_get_school_year(date, schoolyear):
    date = parse_date(date)
    assert schoolyear == get_school_year(date)


@pytest.mark.parametrize('date, monday, sunday', [
    ('2020-12-27', '2020-12-21', '2020-12-27'),
    ('2020-12-28', '2020-12-28', '2021-01-03'),
    ('2020-12-31', '2020-12-28', '2021-01-03'),
    ('2021-01-01', '2020-12-28', '2021-01-03'),
    ('2021-01-03', '2020-12-28', '2021-01-03'),
])
def test_week_boundaries(date, monday, sunday):
    start, end = week_boundaries_datetimes(date)
    assert start.strftime('%Y-%m-%d') == monday
    assert end.strftime('%Y-%m-%d') == sunday


@pytest.mark.parametrize('start, end, items', [
    ('2020-10-22', '2020-12-06', ['2020-10-01', '2020-11-01', '2020-12-01']),
    ('2020-12-31', '2021-01-03', ['2020-12-01', '2021-01-01']),
    ('2020-12-28', '2020-12-28', ['2020-12-01']),
    ('2021-01-03', '2020-12-01', []),
])
def test_month_range(start, end, items):
    start = parse_date(start)
    end = parse_date(end)
    assert [x.strftime('%Y-%m-%d') for x in month_range(start, end)] == items


def test_decompose_event():
    resp = get_json_file('child_planning_before_decomposition')
    data = resp['data']
    assert len(data) == 43
    assert data[22]['text'] == 'JOURNEE'
    assert data[23]['text'] == 'MATIN'
    assert data[24]['text'] == 'MATIN ET REPAS'
    assert data[25]['text'] == 'APRES MIDI'
    assert not [x for x in data if x['text'] == 'REPAS']

    # unit is break down into its components
    assert [x['text'] for x in decompose_event(data[22])] == [
        'Matinée', 'Repas', 'Après-midi']
    assert [x['text'] for x in decompose_event(data[23])] == [
        'Matinée']
    assert [x['text'] for x in decompose_event(data[24])] == [
        'Matinée', 'Repas']
    assert [x['text'] for x in decompose_event(data[25])] == [
        'Après-midi']

    # child_planning function use a dict to remove dupplicated components
    data = {x['id']: x for e in data for x in decompose_event(e)}.values()
    assert len(data) == 42
    assert len([x for x in data if x['text'] == 'Repas']) == 1
    assert len([x for x in data if 'EO0001' in x['slot_id']]) == 3


@override_settings(TIME_ZONE='Europe/Paris')
@pytest.mark.parametrize('legacy, nb_events, nb_booked, response', [
    ('please', 43, 9, 'child_planning_before_decomposition'),
    ('', 42, 9, 'child_planning_after_decomposition'),
])
@mock.patch('passerelle.utils.Request.get')
@mock.patch('passerelle.utils.Request.post')
def test_get_child_planning(mocked_post, mocked_get, legacy, nb_events, nb_booked, response,
                            family_service_wsdl, activity_service_wsdl, app, connector):
    mocked_get.side_effect = (
        utils.FakedResponse(
            content=family_service_wsdl,
            status_code=200,
            headers={'Content-Type': 'text/xml'}),
        utils.FakedResponse(
            content=activity_service_wsdl,
            status_code=200,
            headers={'Content-Type': 'text/xml'}),
        utils.FakedResponse(
            content=activity_service_wsdl,
            status_code=200,
            headers={'Content-Type': 'text/xml'})
    )
    mocked_post.side_effect = (
        utils.FakedResponse(
            content=get_xml_file('readFamily.xml'),
            status_code=200,
            headers={'Content-Type': 'text/xml'}),
        utils.FakedResponse(
            content=get_xml_file('child_planning_readActivityListResponse.xml'),
            status_code=200,
            headers={'Content-Type': 'text/xml'}),
        utils.FakedResponse(
            content=get_xml_file('child_planning_readChildMonthPlanningResponse.xml'),
            status_code=200,
            headers={'Content-Type': 'text/xml'})
    )
    Link.objects.create(resource=connector, family_id='3264', name_id='local')
    url = '/maelis/test/child-planning?NameID=local&childID=21293&start_date=2020-12-19'
    url += '&legacy=' + legacy
    resp = app.get(url)
    data = resp.json['data']
    previous_event = {}
    for event in data:
        assert event['category']
        assert event['text']
        if previous_event:
            assert previous_event['id'] <= event['id']
        previous_event = event
    assert len(data) == nb_events
    assert len([s for s in data if s['user_booking_status'] == 'booked']) == nb_booked
    assert resp.json == get_json_file(response)
